<?php

use Ali2Woo\Aliexpress;
use Ali2Woo\AliexpressHelper;
use Ali2Woo\ApplyPricingRulesProcess;
use Ali2Woo\Attachment;
use Ali2Woo\BackgroundProcessFactory;
use Ali2Woo\BackgroundProcessService;
use Ali2Woo\ExternalOrderFactory;
use Ali2Woo\FulfillmentClient;
use Ali2Woo\GlobalSystemMessageService;
use Ali2Woo\Helper;
use Ali2Woo\ImportAjaxController;
use Ali2Woo\ImportListService;
use Ali2Woo\ImportProcess;
use Ali2Woo\OrderFulfillmentService;
use Ali2Woo\Override;
use Ali2Woo\PermanentAlertService;
use Ali2Woo\PriceFormulaFactory;
use Ali2Woo\PriceFormulaRepository;
use Ali2Woo\PriceFormulaService;
use Ali2Woo\PriceFormulaSetAjaxController;
use Ali2Woo\PriceFormulaSetFactory;
use Ali2Woo\PriceFormulaSetRepository;
use Ali2Woo\PriceFormulaSetService;
use Ali2Woo\PriceFormulaSettingsRepository;
use Ali2Woo\ProductChange;
use Ali2Woo\ProductImport;

use Ali2Woo\Review;
use Ali2Woo\SplitProductService;
use Ali2Woo\TipOfDayAjaxController;
use Ali2Woo\TipOfDayFactory;
use Ali2Woo\TipOfDayRepository;
use Ali2Woo\TipOfDayService;
use Ali2Woo\Woocommerce;
use function DI\create;
use function DI\get;
use Ali2Woo\Settings;

return [
    /* helpers */
    'Ali2Woo\AliexpressHelper' => create(AliexpressHelper::class),

    /* apis */
    'Ali2Woo\FulfillmentClient' => create(FulfillmentClient::class),

    /* factories */
    'Ali2Woo\BackgroundProcessFactory' => create(BackgroundProcessFactory::class),
    'Ali2Woo\ExternalOrderFactory' => create(ExternalOrderFactory::class),
    'Ali2Woo\PriceFormulaFactory' => create(PriceFormulaFactory::class),
    'Ali2Woo\PriceFormulaSetFactory' => create(PriceFormulaSetFactory::class)
        ->constructor(
            get(PriceFormulaFactory::class),
        ),
    'Ali2Woo\TipOfDayFactory' => create(TipOfDayFactory::class),

    /* repository */
    'Ali2Woo\PriceFormulaRepository' => create(PriceFormulaRepository::class)
        ->constructor(
            get(PriceFormulaFactory::class)
        ),
    'Ali2Woo\PriceFormulaSetRepository' => create(PriceFormulaSetRepository::class)
        ->constructor(
            get(PriceFormulaSetFactory::class)
        ),
    'Ali2Woo\TipOfDayRepository' => create(TipOfDayRepository::class)
        ->constructor(
            get(TipOfDayFactory::class)
        ),

    /* models */
    'Ali2Woo\Attachment' => create(Attachment::class),
    'Ali2Woo\Helper' => create(Helper::class),
    'Ali2Woo\ProductChange' => create(ProductChange::class),
    'Ali2Woo\ProductImport' => create(ProductImport::class),
    'Ali2Woo\Woocommerce' => create(Woocommerce::class)
        ->constructor(
            get(Attachment::class), get(Helper::class), get(ProductChange::class)
        ),
    'Ali2Woo\Review' => create(Review::class),
    'Ali2Woo\Override' => create(Override::class),
    'Ali2Woo\Aliexpress' => create(Aliexpress::class),

    /* services */
    'Ali2Woo\BackgroundProcessService' => create(BackgroundProcessService::class)
        ->constructor(get(ApplyPricingRulesProcess::class), get(ImportProcess::class)),
    'Ali2Woo\PermanentAlertService' => create(PermanentAlertService::class)
        ->constructor(get(BackgroundProcessService::class)),
    'Ali2Woo\ImportListService' => create(ImportListService::class)
        ->constructor(get(ProductImport::class), get(Aliexpress::class)),
    'Ali2Woo\OrderFulfillmentService' => create(OrderFulfillmentService::class)
        ->constructor(get(Aliexpress::class), get(ExternalOrderFactory::class)),
    'Ali2Woo\PriceFormulaService' => create(PriceFormulaService::class)
        ->constructor(
            get(PriceFormulaRepository::class),
            get(PriceFormulaSettingsRepository::class)
        ),
    'Ali2Woo\PriceFormulaSetService' => create(PriceFormulaSetService::class)
        ->constructor(
            get(PriceFormulaRepository::class),
            get(PriceFormulaSettingsRepository::class),
            get(PriceFormulaSetFactory::class),
            get(BackgroundProcessFactory::class)
        ),
    'Ali2Woo\SplitProductService' => create(SplitProductService::class)
        ->constructor(
            get(ProductImport::class),
        ),
    'Ali2Woo\GlobalSystemMessageService' => create(GlobalSystemMessageService::class),
    'Ali2Woo\TipOfDayService' => create(TipOfDayService::class)
        ->constructor(
            get(TipOfDayFactory::class),
            get(TipOfDayRepository::class),
            Settings::instance()
        ),
    
    /* controllers */
    'Ali2Woo\ImportAjaxController' => create(ImportAjaxController::class)
        ->constructor(
            get(ProductImport::class), get(Woocommerce::class), get(Review::class),
            get(Override::class), get(Aliexpress::class), get(SplitProductService::class)
        ),

    'Ali2Woo\PriceFormulaSetAjaxController' => create(PriceFormulaSetAjaxController::class)
        ->constructor(
            get(PriceFormulaSetRepository::class),
            get(PriceFormulaSetService::class)
        ),
    'Ali2Woo\TipOfDayAjaxController' => create(TipOfDayAjaxController::class)
        ->constructor(
            get(TipOfDayService::class),
            get(TipOfDayRepository::class),
        )
];
