<?php
/**
 * Description of ProductDataTabController
 *
 * @author Ali2Woo Team
 *
 * @autoload: a2w_admin_init
 *
 * @ajax: true
 */

// phpcs:ignoreFile WordPress.Security.NonceVerification.Recommended
namespace Ali2Woo;

class ProductDataTabController extends AbstractController
{

    public $tab_class = '';
    public $tab_id = '';
    public $tab_title = '';
    public $tab_icon = '';

    public function __construct() {
        parent::__construct();
        $this->tab_class = 'a2w_product_data';
        $this->tab_id = 'a2w_product_data';
        $this->tab_title = 'A2W Data';

        add_action('admin_head', array(&$this, 'on_admin_head'));
        add_action('woocommerce_product_write_panel_tabs', array(&$this, 'product_write_panel_tabs'), 99);
        add_action('woocommerce_product_data_panels', [$this, 'product_data_panel_wrap'], 99);
        add_action('woocommerce_process_product_meta', [$this, 'process_product_meta'], 1, 2);
        add_action('woocommerce_variation_options_pricing', [$this, 'variation_options_pricing'], 20, 3);

        add_action('wp_ajax_a2w_data_remove_deleted_attribute', [$this, 'ajax_remove_deleted_attribute']);
        add_action('wp_ajax_a2w_data_remove_deleted_variation', [$this, 'ajax_remove_deleted_variation']);
        add_action('wp_ajax_a2w_data_last_update_clean', [$this, 'ajax_last_update_clean']);
        add_action('wp_ajax_a2w_update_product_shipping_info', [$this, 'ajax_update_product_shipping_info']);
        add_action('wp_ajax_a2w_remove_product_shipping_info', [$this, 'ajax_remove_product_shipping_info']);
    }

    public function on_admin_head() {
        echo '<style type="text/css">#woocommerce-product-data ul.wc-tabs li.' . $this->tab_class . ' a::before {content: \'\f163\';}</style>';
    }

    public function product_write_panel_tabs() {
        ?>
        <li class="<?php echo $this->tab_class; ?>"><a href="#<?php echo $this->tab_id; ?>"><span><?php echo $this->tab_title; ?></span></a></li>
        <?php
    }

    public function product_data_panel_wrap(): void
    {
        ?>
        <div id="<?php echo $this->tab_id; ?>" class="panel <?php echo $this->tab_class; ?> woocommerce_options_panel wc-metaboxes-wrapper" style="display:none">
            <?php $this->render_product_tab_content(); ?>
        </div>
        <?php
    }

    private function render_product_tab_content(): void
    {
        global $post;

        $post_id = $_REQUEST['post'] ?? "";

        $country_model = new Country();

        $this->model_put('post_id', $post_id);
        $this->model_put('countries', $country_model->get_countries());

        $this->include_view("product_data_tab.php");
    }

    public function process_product_meta($post_id, $post): void
    {
        if (isset($_POST['_a2w_external_id'])) {
            update_post_meta($post_id, '_a2w_external_id', $_POST['_a2w_external_id']);
        } else {
            delete_post_meta($post_id, '_a2w_external_id');
        }

        if (isset($_POST['_a2w_orders_count'])) {
            update_post_meta($post_id, '_a2w_orders_count', $_POST['_a2w_orders_count']);
        } else {
            delete_post_meta($post_id, '_a2w_orders_count');
        }

        update_post_meta($post_id, '_a2w_disable_sync', !empty($_POST['_a2w_disable_sync']) ? 1 : 0);

        update_post_meta($post_id, '_a2w_disable_var_price_change', !empty($_POST['_a2w_disable_var_price_change']) ? 1 : 0);

        update_post_meta($post_id, '_a2w_disable_var_quantity_change', !empty($_POST['_a2w_disable_var_quantity_change']) ? 1 : 0);

        update_post_meta($post_id, '_a2w_disable_add_new_variants', !empty($_POST['_a2w_disable_add_new_variants']) ? 1 : 0);

        if (!empty($_POST['_a2w_last_update'])) {
            update_post_meta($post_id, '_a2w_last_update', $_POST['_a2w_last_update']);
        } else {
            delete_post_meta($post_id, '_a2w_last_update');
        }
        if (!empty($_POST['_a2w_reviews_last_update'])) {
            update_post_meta($post_id, '_a2w_reviews_last_update', $_POST['_a2w_reviews_last_update']);
        } else {
            delete_post_meta($post_id, '_a2w_reviews_last_update');
        }
        if (!empty($_POST['_a2w_review_page'])) {
            update_post_meta($post_id, '_a2w_review_page', $_POST['_a2w_review_page']);
        } else {
            delete_post_meta($post_id, '_a2w_review_page');
        }
    }

    public function variation_options_pricing($loop, $variation_data, $variation): void
    {
        if (!empty($variation_data['_aliexpress_regular_price']) || !empty($variation_data['_aliexpress_price'])) {

            $helpTip = esc_html__('Source Regular price loaded from Aliexpress', 'ali2woo');
            $label = sprintf(esc_html__('Aliexpress Regular price (%s)', 'ali2woo'), get_woocommerce_currency_symbol());
            $value = wc_format_localized_price(is_array($variation_data['_aliexpress_regular_price']) ? $variation_data['_aliexpress_regular_price'][0] : $variation_data['_aliexpress_regular_price']);
            $this->outputSettingRow($label, $value, "form-row-first", $helpTip);

            $helpTip = esc_html__('Source Sale price loaded from Aliexpress', 'ali2woo');
            $label = sprintf(esc_html__('Aliexpress Sale price (%s)', 'ali2woo'), get_woocommerce_currency_symbol());
            $value = wc_format_localized_price(is_array($variation_data['_aliexpress_price']) ? $variation_data['_aliexpress_price'][0] : $variation_data['_aliexpress_price']);
            $this->outputSettingRow($label, $value, "form-row-last", $helpTip);

            $notLoadedText = esc_html__('not loaded', 'ali2woo');

            $helpTip = esc_html__('Source Sku ID loaded from Aliexpress', 'ali2woo');
            $label = esc_html__('Aliexpress Sku ID', 'ali2woo');
            $value = !empty($variation_data['_a2w_ali_sku_id'][0]) ? $variation_data['_a2w_ali_sku_id'][0] : $notLoadedText;
            $this->outputSettingRow($label, $value, "form-row-first", $helpTip);

            $helpTip = esc_html__('Source Sku properties loaded from Aliexpress', 'ali2woo');
            $label = esc_html__('Aliexpress Sku Props', 'ali2woo');
            $value = !empty($variation_data['_aliexpress_sku_props'][0]) ? $variation_data['_aliexpress_sku_props'][0] : $notLoadedText;
            $this->outputSettingRow($label, $value, "form-row-last", $helpTip);

            $helpTip = esc_html__('All attributes included to this variation', 'ali2woo');
            $label = esc_html__('Attributes', 'ali2woo');
            $values = [];
            foreach ($variation_data as $itemKey => $dataItem) {
                if (str_starts_with($itemKey, 'attribute_pa_')) {
                    $attrLabel = str_replace('attribute_pa_', '', $itemKey);
                    $values[] = $attrLabel . ': ' .  (!empty($dataItem[0]) ? $dataItem[0] : '');
                }
            }
            $value = implode('; ', $values);
            $this->outputSettingRow($label, $value, "", $helpTip);
        }
    }

    public function ajax_remove_deleted_attribute(): void
    {
        check_admin_referer(self::AJAX_NONCE_ACTION, self::NONCE);

        if (!current_user_can('manage_options')) {
            $result = ResultBuilder::buildError($this->getErrorTextNoPermissions());
            echo wp_json_encode($result);
            wp_die();
        }

        if (!empty($_POST['post_id']) && !empty($_POST['id'])) {
            $deleted_variations_attributes = get_post_meta($_POST['post_id'], '_a2w_deleted_variations_attributes', true);
            if ($deleted_variations_attributes) {
                foreach ($deleted_variations_attributes as $k => $a) {
                    if ($_POST['id'] == 'all' || $k == sanitize_title($_POST['id'])) {
                        unset($deleted_variations_attributes[$k]);
                    }
                }
            }
            update_post_meta($_POST['post_id'], '_a2w_deleted_variations_attributes', $deleted_variations_attributes);
        }

        echo wp_json_encode(ResultBuilder::buildOk());
        wp_die();
    }

    public function ajax_remove_deleted_variation(): void
    {
        check_admin_referer(self::AJAX_NONCE_ACTION, self::NONCE);

        if (!current_user_can('manage_options')) {
            $result = ResultBuilder::buildError($this->getErrorTextNoPermissions());
            echo wp_json_encode($result);
            wp_die();
        }

        if (!empty($_POST['post_id'])) {
            $a2w_skip_meta = get_post_meta($_POST['post_id'], "_a2w_skip_meta", true);
            $a2w_skip_meta = $a2w_skip_meta?$a2w_skip_meta:array('skip_vars' => array(), 'skip_images' => array());
            if ($_POST['id']=='all') {
                $a2w_skip_meta['skip_vars'] = array();
            } else {
                $a2w_skip_meta['skip_vars'] = array_filter(array_diff($a2w_skip_meta['skip_vars'], array($_POST['id'])));
            }
            update_post_meta($_POST['post_id'], "_a2w_skip_meta", $a2w_skip_meta);
        }

        echo wp_json_encode(ResultBuilder::buildOk());
        wp_die();
    }

    public function ajax_last_update_clean(): void
    {
        check_admin_referer(self::AJAX_NONCE_ACTION, self::NONCE);

        if (!current_user_can('manage_options')) {
            $result = ResultBuilder::buildError($this->getErrorTextNoPermissions());
            echo wp_json_encode($result);
            wp_die();
        }

        if (!empty($_POST['post_id']) && !empty($_POST['type'])) {
            if ($_POST['type'] === 'product') {
                delete_post_meta($_POST['post_id'], '_a2w_last_update');
            } else if($_POST['type'] === 'review') {
                delete_post_meta($_POST['post_id'], '_a2w_reviews_last_update');
            }
        }

        echo wp_json_encode(ResultBuilder::buildOk());
        wp_die();
    }

    public function ajax_update_product_shipping_info(): void
    {
        check_admin_referer(self::AJAX_NONCE_ACTION, self::NONCE);

        if (!current_user_can('manage_options')) {
            $result = ResultBuilder::buildError($this->getErrorTextNoPermissions());
            echo wp_json_encode($result);
            wp_die();
        }

        if (!empty($_POST['id']) && isset($_POST['cost']) && !empty($_POST['country_to']) && !empty($_POST['method']) && !empty($_POST['items'])) {
            $shipping_meta = new ProductShippingMeta($_POST['id']);
            $shipping_meta->save_cost($_POST['cost'], false);
            $shipping_meta->save_method($_POST['method'], false);

            $shipping_meta->save_country_to($_POST['country_to'], false);
            if (isset($_POST['country_from'])) {
                $shipping_meta->save_country_from($_POST['country_from'], false);
                $shipping_meta->save_items(1, $_POST['country_from'], $_POST['country_to'], $_POST['items'], false);
            } else {
                $shipping_meta->save_items(1, '', $_POST['country_to'], $_POST['items'], false);
            }

            $shipping_meta->save();
            echo wp_json_encode(ResultBuilder::buildOk());
        } else {
            echo wp_json_encode(ResultBuilder::buildError('wrong params'));
        }

        wp_die();
    }

    public function ajax_remove_product_shipping_info(): void
    {
        check_admin_referer(self::AJAX_NONCE_ACTION, self::NONCE);

        if (!current_user_can('manage_options')) {
            $result = ResultBuilder::buildError($this->getErrorTextNoPermissions());
            echo wp_json_encode($result);
            wp_die();
        }

        if (!empty($_POST['id'])) {
            $shipping_meta = new ProductShippingMeta($_POST['id']);
            $shipping_meta->save_cost('', false);
            $shipping_meta->save_country_to('', false);
            $shipping_meta->save_method('', false);
            $shipping_meta->save();
            echo wp_json_encode(ResultBuilder::buildOk());
        } else {
            echo wp_json_encode(ResultBuilder::buildError('wrong params'));
        }

        wp_die();
    }

    private function outputSettingRow(string $label, string $value, string $class = "", string $helpTip = ""): void
    {
        echo '<p class="form-field form-row ' . $class . '">';
        if (!empty($value)) {
            echo '<label style="cursor: inherit;">' . $label . ':</label>&nbsp;<label style="cursor: inherit;">' . $value . '</label>';
        }
        if (!empty($helpTip)) {
            echo '<span class="woocommerce-help-tip" data-tip="' . $helpTip . '"></span>';
        }
        echo '</p>';
    }
}
